<?php
/*
Plugin Name: Category Levels
Plugin URI: http://www.coppit.org/code/
Description: Protects any post in a category whose required level is greater than the user's level.
Version: 0.3.2
Author: David Coppit
Author URI: http://www.coppit.org
*/

// --------------------------------------------------------------------

global $default_private_message;
$default_private_message =
  'Sorry, you do not have sufficient privileges to view this post.';

add_filter('list_cats','CL_filter_category_list_item');
add_filter('list_cats_exclusions','CL_filter_category_list');
add_filter('the_title','CL_error_text');
add_filter('the_title','CL_post_padlock');
add_filter('the_content','CL_hide_text');
add_filter('single_post_title','CL_error_text');
add_filter('comment_url','CL_hide_text');
add_filter('comment_excerpt','CL_hide_text');
add_filter('comment_email','CL_hide_text');
add_filter('comment_author','CL_hide_text');
add_filter('comment_text','CL_hide_text');
add_filter('the_title_rss','CL_error_text');
add_filter('the_excerpt','CL_hide_text');
add_filter('the_posts','CL_filter_posts');

add_action('admin_menu', 'CL_setup_category_level_management_page');

// --------------------------------------------------------------------

function CL_setup_category_level_management_page() {
  if (function_exists('add_management_page'))
    add_management_page('Category Levels', 'Category Levels',
      9, __FILE__, 'CL_manage_category_levels');
}

// Main implementation of the management page
function CL_manage_category_levels() {
  CL_handle_action();

  print "<div class='wrap'>\n";
  print "<h2> Manage Category Levels </h2>\n";

  // Category table
  print "<h3>Category Levels</h3>\n";

  print "<p> Set the minimum required user level for each category ".
    "below</p>\n";

  $page_only_uri = $_SERVER["REQUEST_URI"];
  $page_only_uri = preg_replace('/&.*/','',$page_only_uri);

  {
    print "<form method='post' action='$page_only_uri'>\n";
    print "<table cellpadding=\"3\" cellspacing=\"3\">\n";

    print "<tr>\n<th>Category</th><th>Level</th>\n</tr>\n";

    $category_levels = CL_get_category_levels();
    $category_lookup = CL_get_category_lookup();

    foreach ($category_lookup as $id => $category) {
      print "<tr>\n";
      print "<td>" . $category->cat_name . "</td>\n";
      print "<td align=center>";

      if ($category_levels[$id] > 0)
        print "<a href=\"$page_only_uri" .
          "&action=demote&id=$id\">-</a>\n";

      print $category_levels[$id];

      print "<a href=\"$page_only_uri" .
        "&action=promote&id=$id\">+</a>\n";

      print "</td>\n";
      print "</tr>\n";
    }

    print "</table>\n";
    print "</form>\n";
  }

  print "<h3>Protected Posts</h3>\n";

  print "<form method='post' action='$page_only_uri'>" .
    "<p><input type=hidden name=action value=toggle_show_private_message>".
    "<input type=checkbox onClick='submit()'";
  if (get_option('Category_Levels_show_private_message'))
    print " checked";
  print ">Show a private message instead of hiding posts.</p></form>\n";


  print "<div style='padding-left:2em";
  if (!get_option('Category_Levels_show_private_message'))
    print ";color:gray;";
  print "'>\n";

  global $default_private_message;
  $private_message = get_option('Category_Levels_private_message');
  if ($private_message == false)
    $private_message = $default_private_message;

  print "<form name=update_message method='post' action='$page_only_uri'>" .
    "<p>The private post message:<br>" .
    "<input type=hidden name=action value=update_private_message />" .
    "<input name=private_message type=text size=50";
  if (!get_option('Category_Levels_show_private_message'))
    print " disabled style='color:gray'";
  print
    " value=\"$private_message\" />" .
    "<input type=submit value='Update'";
  if (!get_option('Category_Levels_show_private_message'))
    print " disabled style='color:gray'";
    " /></p></form>\n";


  print "<form method='post' action='$page_only_uri'>" .
      "<p><input type=hidden name=action value=toggle_show_user_level_on_private_posts>".
      "<input type=checkbox onClick='submit()'";
  if (!get_option('Category_Levels_show_private_message'))
    print " disabled";
  if (get_option('Category_Levels_show_user_level_on_private_posts'))
    print " checked";
  print ">Show the required user level on the private post message.</p></form>\n";

  print "<form method='post' action='$page_only_uri'>" .
    "<p><input type=hidden name=action value=toggle_show_padlock_on_private_posts>".
    "<input type=checkbox onClick='submit()'";
  if (!get_option('Category_Levels_show_private_message'))
    print " disabled";
  if (get_option('Category_Levels_show_padlock_on_private_posts'))
    print " checked";
  print ">Show a padlock icon on the private post message.</p></form>\n";

  print "</div>\n";


  print "<h3>The Category List</h3>\n";


  print "<form method='post' action='$page_only_uri'>" .
    "<p><input type=hidden name=action value=toggle_show_private_categories>".
    "<input type=checkbox onClick='submit()'";
  if (get_option('Category_Levels_show_private_categories'))
    print " checked";
  print ">Show private categories.</p></form>\n";

  print "<div style='padding-left:2em";
  if (!get_option('Category_Levels_show_private_categories'))
    print ";color:gray;";
  print "'>\n";

  print "<form method='post' action='$page_only_uri'>" .
    "<p><input type=hidden name=action value=toggle_show_user_level_on_private_categories>".
    "<input type=checkbox onClick='submit()'";
  if (!get_option('Category_Levels_show_private_categories'))
    print " disabled";
  if (get_option('Category_Levels_show_user_level_on_private_categories'))
    print " checked";
  print ">Show the required user level next to private categories.</p></form>\n";

  print "<form method='post' action='$page_only_uri'>" .
    "<p><input type=hidden name=action value=toggle_show_padlock_on_private_categories>".
    "<input type=checkbox onClick='submit()'";
  if (!get_option('Category_Levels_show_private_categories'))
    print " disabled";
  if (get_option('Category_Levels_show_padlock_on_private_categories'))
    print " checked";
  print ">Show a padlock icon next to private categories.</p></form>\n";

  print "</div>\n";


  print "<form method='post' action='$page_only_uri'>" .
    "<p>".
    "<input type=hidden name=action value=reset_options>".
    "<input type='submit' name='Submit' " .
    "value='Reset All Options'></p></form>\n";

}

// --------------------------------------------------------------------

function CL_handle_action() {
  GLOBAL $_POST, $_GET;

  // Stolen from users.php. Sets the values of the specified variables from
  // the GET or POST object values
  $vars_to_import = array('action','private_message','id');
  for ($i=0; $i<count($vars_to_import); $i += 1) {
    $wpvar = $vars_to_import[$i];
    if (!isset($$wpvar)) {
      if (empty($_POST["$wpvar"])) {
        if (empty($_GET["$wpvar"])) {
          $$wpvar = '';
        } else {
          $$wpvar = $_GET["$wpvar"];
        }
      } else {
        $$wpvar = $_POST["$wpvar"];
      }
    }
  }

  if ($action == 'demote') {
    $category_levels = CL_get_category_levels();
    $category_levels[$id]--;
    update_option('Category_Levels', var_export($category_levels,1));
  }
  else if ($action == 'promote') {
    $category_levels = CL_get_category_levels();
    $category_levels[$id]++;
    update_option('Category_Levels', var_export($category_levels,1));
  }

  else if ($action == 'toggle_show_private_message')
    if (get_option('Category_Levels_show_private_message'))
      update_option('Category_Levels_show_private_message',false);
    else
      update_option('Category_Levels_show_private_message',true);


  else if ($action == 'update_private_message')
    update_option('Category_Levels_private_message',
      htmlentities(stripslashes($private_message),ENT_QUOTES));


  else if ($action == 'toggle_show_user_level_on_private_posts')
    if (get_option('Category_Levels_show_user_level_on_private_posts'))
      update_option('Category_Levels_show_user_level_on_private_posts',false);
    else
      update_option('Category_Levels_show_user_level_on_private_posts',true);


  else if ($action == 'toggle_show_padlock_on_private_posts')
    if (get_option('Category_Levels_show_padlock_on_private_posts'))
      update_option('Category_Levels_show_padlock_on_private_posts',false);
    else
      update_option('Category_Levels_show_padlock_on_private_posts',true);


  if ($action == 'toggle_show_private_categories')
    if (get_option('Category_Levels_show_private_categories'))
      update_option('Category_Levels_show_private_categories',false);
    else
      update_option('Category_Levels_show_private_categories',true);


  else if ($action == 'toggle_show_user_level_on_private_categories')
    if (get_option('Category_Levels_show_user_level_on_private_categories'))
      update_option('Category_Levels_show_user_level_on_private_categories',false);
    else
      update_option('Category_Levels_show_user_level_on_private_categories',true);


  else if ($action == 'toggle_show_padlock_on_private_categories')
    if (get_option('Category_Levels_show_padlock_on_private_categories'))
      update_option('Category_Levels_show_padlock_on_private_categories',false);
    else
      update_option('Category_Levels_show_padlock_on_private_categories',true);


  else if ($action == 'reset_options') {
    // I don't know why delete_option doesn't work for CL_get_category_levels
    // later. update_option with an empty array works.
    update_option('Category_Levels', var_export(array(),1));

    update_option('Category_Levels_private_message',false);
    update_option('Category_Levels_show_private_message',false);
    update_option('Category_Levels_show_user_level_on_private_posts',false);
    update_option('Category_Levels_show_padlock_on_private_posts',false);

    update_option('Category_Levels_show_private_categories',false);
    update_option('Category_Levels_show_user_level_on_private_categories', false);
    update_option('Category_Levels_show_padlock_on_private_categories', false);
  }

}

// --------------------------------------------------------------------

// Returns a hash from IDs to associated category objects
function CL_get_category_lookup() {
  global $wpdb;

  $categories =
    $wpdb->get_results("SELECT * FROM $wpdb->categories ORDER BY cat_name");

  $category_lookup = array();

  foreach ($categories as $category)
    $category_lookup[$category->cat_ID] = $category;

  return $category_lookup;
}

// --------------------------------------------------------------------

function CL_get_category_levels() {
  $serialized_category_levels = get_option('Category_Levels');

  if ($serialized_category_levels == false)
    $category_levels = array();
  else
    eval("\$category_levels = $serialized_category_levels;");

  $category_lookup = CL_get_category_lookup();

  // Remove any references to unknown categories
  foreach (array_keys($category_levels) as $id)
    if ( !array_key_exists($id, $category_lookup) )
      unset($category_levels[$id]);

  // Add unreferenced categories to the end of the top level
  foreach (array_keys($category_lookup) as $id)
    if ( !array_key_exists($id, $category_levels) )
      $category_levels[$id] = 0;

  return $category_levels;
}

// --------------------------------------------------------------------

// Returns -1 if the user level can't be determined.
function CL_required_user_level($post)
{
  $category_levels = CL_get_category_levels();

  if ($post->ID == null)
    return -1;

  $post_categories = wp_get_post_cats(1, $post->ID);

  $required_user_level = 0;

  foreach ($post_categories as $post_category_id)
  {
    if ($category_levels[$post_category_id] > $required_user_level)
      $required_user_level = $category_levels[$post_category_id];
  }

  return $required_user_level;
}

// --------------------------------------------------------------------

function CL_should_be_hidden($post = 0)
{
    if (!$post)
      global $post;

    global $user_level;

    if (intval($user_level) < CL_required_user_level($post))
      return 1;

    return 0;
}

// --------------------------------------------------------------------

function CL_filter_category_list_item($text)
{
  // In case this filter was called with the whole list. (In this case,
  // $category_name is really the whole list, and we don't want to change it.)
  // This detection is a hack, since I don't know how to do it right.
  if (strpos($text,"\n") || strpos($text,"<\/a>"))
    return $text;

  global $user_level;

  $category_name = $text;
  $category_levels = CL_get_category_levels();
  $category_lookup = CL_get_category_lookup();

  foreach ($category_lookup as $id => $category) {
    if ($category->cat_name != $category_name)
      continue;

    if (intval($user_level) >= $category_levels[$id])
      return $text;

    if (get_option('Category_Levels_show_padlock_on_private_categories'))
    {
      $site_root = parse_url(get_settings('siteurl'));
      $site_root = trailingslashit($site_root['path']);

      $text .= ' <img src=\'' . $site_root .
        'wp-content/plugins/category-levels/padlock.gif\' ' .
        'height=\'10\' width=\'8\' valign=\'middle\' border=\'0\'/>';
    }

    if (get_option('Category_Levels_show_user_level_on_private_categories'))
      $text .= '<sup>' . $category_levels[$id] . '</sup>';

    return $text;
  }

  // We really should never reach here
  assert(false);
  return $text;
}
// --------------------------------------------------------------------

function CL_filter_category_list($exclusions)
{
  global $user_level;

  if (get_option('Category_Levels_show_private_categories'))
    return $exclusions;

  $category_levels = CL_get_category_levels();
  $category_lookup = CL_get_category_lookup();

  foreach ($category_lookup as $id => $category) {
    if (!array_key_exists($id, $category_levels))
      continue;

    if (intval($user_level) < $category_levels[$id])
      $exclusions .= " AND cat_ID <> $id ";
  }

  return $exclusions;
}

// --------------------------------------------------------------------

function CL_filter_posts($allposts)
{
  global $feed;

  if ( (empty($feed) && get_option('Category_Levels_show_private_message')) ||
      !is_array($allposts) )
    return $allposts;

  foreach ($allposts as $post)
  {
    if (!CL_should_be_hidden($post))
      $goodposts[] = $post;
  }

  return $goodposts;
}

// --------------------------------------------------------------------

function CL_post_padlock($text)
{
  global $post;

  if (!CL_should_be_hidden($post))
    return $text;

  if (get_option('Category_Levels_show_padlock_on_private_posts') &&
    CL_required_user_level($post))
  {
    $site_root = parse_url(get_settings('siteurl'));
    $site_root = trailingslashit($site_root['path']);

    $text = '<img src=\'' . $site_root .
      'wp-content/plugins/category-levels/padlock.gif\' ' .
      'valign=\'middle\' border=\'0\'/>' . $text;
  }

  if (get_option('Category_Levels_show_user_level_on_private_posts'))
    $text .= ' (Level ' . CL_required_user_level($post) . ' required)';

  return $text;
}

// --------------------------------------------------------------------

function CL_hide_text($text)
{
  if (CL_should_be_hidden())
    $text = '';

  return $text;
}

// --------------------------------------------------------------------

function CL_error_text($text)
{
  global $default_private_message;

  if (CL_should_be_hidden()) {
    if (get_option('Category_Levels_private_message'))
      $text = get_option('Category_Levels_private_message');
    else
      $text = $default_private_message;
  }

  return $text;
}

// Used during development to dump a data structure to html
function CL_print_html_data($data) {
  $string = htmlspecialchars(print_r($data,1));
  $string = preg_replace("/\n/", "<br>\n", $string);
  $string = preg_replace("/ /", "&nbsp;", $string);

  print $string;
}

?>
